import React, { Component } from 'react';

import { dieHardMachine } from './dieHardMachine';
import { interpret } from 'xstate/lib/interpreter';
import styled from 'styled-components';

const StyledApp = styled.main`
  display: flex;
  flex-direction: row;
  justify-content: space-between;
  align-items: flex-end;
`;

const StyledJug = styled.div`
  width: 20vmin;
  height: calc(var(--volume) * 10vmin);
  border: 1px solid white;
  margin: 0 5vmin;
  display: flex;
  flex-direction: column;
  justify-content: flex-end;
  align-items: center;

  &:before {
    content: '';
    position: absolute;
    bottom: 0;
    left: 0;
    width: 100%;
    height: calc(var(--amount) * 10vmin);
    background: blue;
  }
`;

function Jug({ volume, amount, onEmpty, onFill, onTransfer }) {
  return (
    <StyledJug
      style={{
        '--volume': volume,
        '--amount': amount
      }}
    >
      <button onClick={onEmpty}>Empty</button>
      <button onClick={onFill}>Fill</button>
      <button onClick={onTransfer}>Transfer</button>
    </StyledJug>
  );
}

export class DieHard extends React.Component {
  state = {
    current: dieHardMachine.initialState
  };
  service = interpret(dieHardMachine)
    .onTransition(state => {
      this.setState({ current: state });
    })
    .onEvent(e => console.log(e));
  componentDidMount() {
    this.service.start();
  }
  render() {
    const { send } = this.service;
    const { context } = this.state.current;

    return (
      <StyledApp>
        <Jug
          volume={3}
          amount={context['3']}
          onEmpty={() => send('EMPTY_3')}
          onFill={() => send('FILL_3')}
          onTransfer={() => send('POUR_3_TO_5')}
        />
        <Jug
          volume={5}
          amount={context['5']}
          onEmpty={() => send('EMPTY_5')}
          onFill={() => send('FILL_5')}
          onTransfer={() => send('POUR_5_TO_3')}
        />
      </StyledApp>
    );
  }
}
