const { Machine, actions } = require('xstate');
const { assign } = actions;
const {
  getShortestValuePaths,
  deserializeStateString
} = require('xstate/lib/graph');

export const dieHardMachine = Machine(
  {
    initial: 'pending',
    states: {
      pending: {
        on: {
          '': {
            target: 'success',
            cond: 'weHave4Gallons'
          },
          POUR_3_TO_5: {
            actions: 'pour3to5'
          },
          POUR_5_TO_3: {
            actions: 'pour5to3'
          },
          FILL_3: {
            actions: 'fill3'
          },
          FILL_5: {
            actions: 'fill5'
          },
          EMPTY_3: {
            actions: 'empty3'
          },
          EMPTY_5: {
            actions: 'empty5'
          }
        }
      },
      success: {}
    }
  },
  {
    actions: {
      pour3to5: assign((ctx, e) => {
        const poured = Math.min(5 - ctx[5], ctx[3]);

        return {
          3: ctx[3] - poured,
          5: ctx[5] + poured
        };
      }),
      pour5to3: assign((ctx, e) => {
        const poured = Math.min(3 - ctx[3], ctx[5]);

        return {
          3: ctx[3] + poured,
          5: ctx[5] - poured
        };
      }),
      fill3: assign({ 3: 3 }),
      fill5: assign({ 5: 5 }),
      empty3: assign({ 3: 0 }),
      empty5: assign({ 5: 0 })
    },
    guards: {
      weHave4Gallons: ctx => ctx[5] === 4
    }
  },
  { 3: 0, 5: 0 }
);

// const shortestValuePaths = getShortestValuePaths(dieHardMachine, {
//   events: {
//     POUR_3_TO_5: [{ type: 'POUR_3_TO_5' }],
//     POUR_5_TO_3: [{ type: 'POUR_5_TO_3' }],
//     FILL_3: [{ type: 'FILL_3' }],
//     FILL_5: [{ type: 'FILL_5' }],
//     EMPTY_3: [{ type: 'EMPTY_3' }],
//     EMPTY_5: [{ type: 'EMPTY_5' }]
//   }
// });

// Object.keys(shortestValuePaths).forEach(key => {
//   const state = deserializeStateString(key);
//   console.log(state, shortestValuePaths[key]);
// });
