var toString = Object.prototype.toString;
export function cloneDeep(value) {
    return cloneDeepHelper(value, new Map());
}
function cloneDeepHelper(val, seen) {
    switch (toString.call(val)) {
        case "[object Array]": {
            if (seen.has(val))
                return seen.get(val);
            var copy_1 = val.slice(0);
            seen.set(val, copy_1);
            copy_1.forEach(function (child, i) {
                copy_1[i] = cloneDeepHelper(child, seen);
            });
            return copy_1;
        }
        case "[object Date]":
            return new Date(+val);
        case "[object Object]": {
            if (seen.has(val))
                return seen.get(val);
            var copy_2 = Object.create(Object.getPrototypeOf(val));
            seen.set(val, copy_2);
            if (typeof Object.getOwnPropertyDescriptor === "function") {
                var handleKey = function (key) {
                    var desc = Object.getOwnPropertyDescriptor(val, key);
                    desc.value = cloneDeepHelper(val[key], seen);
                    if (desc.get)
                        delete desc.get;
                    if (desc.set)
                        delete desc.set;
                    Object.defineProperty(copy_2, key, desc);
                };
                Object.getOwnPropertyNames(val).forEach(handleKey);
                if (typeof Object.getOwnPropertySymbols === "function") {
                    Object.getOwnPropertySymbols(val).forEach(handleKey);
                }
            }
            else {
                Object.keys(val).forEach(function (key) {
                    copy_2[key] = cloneDeepHelper(val[key], seen);
                });
            }
            return copy_2;
        }
        default:
            return val;
    }
}
//# sourceMappingURL=cloneDeep.js.map